#include "iic_master.h"

static eIicStatus_t iicStatus = eIicStatus_free;

/*
    Init
*/
void IicMaster_Init(Iic_Master_t *instance, uint8_t addr)
{
    instance->mode = eMasterMode_standby;
    instance->addr = addr;
    IicSender_Init(&instance->sender);
    IicReceiver_Init(&instance->receiver);
}

/*
    Send
*/
eIicRespond_t IicMaster_Send(
    Iic_Master_t *instance,
    uint8_t *buff,
    uint16_t size)
{
    eIicRespond_t resp = eIicRespond_busy;

    switch (instance->mode) {
        case eMasterMode_standby:
            if (iicStatus == eIicStatus_free) {
                iicStatus = eIicStatus_busy;
                instance->mode = eMasterMode_send;
                instance->sender.buff = buff;
                instance->sender.buffSize = size;        
                instance->sender.mode = eSendMode_send;
            }
            break;
        case eMasterMode_send:
            if (instance->sender.mode >= eSendMode_end) {
                if (instance->sender.mode == eSendMode_end) {
                    resp = eIicRespond_end;
                } else {
                    resp = eIicRespond_err;
                }
                iicStatus = eIicStatus_free;
                instance->mode = eMasterMode_standby;
                IicSender_Init(&instance->sender);
            }
            break;
        default:
            break;
    }

    return (resp);
}

/*
    Operation
*/
void IicMaster_Operate(Iic_Master_t *instance)
{
    switch (instance->mode) {
        default:
        case eMasterMode_standby:
            IicSender_Init(&instance->sender);
            IicReceiver_Init(&instance->receiver);
            break;
        case eMasterMode_send:
            IicSender_Operate(
                &instance->sender,
                instance->addr);
            break;
        case eMasterMode_receive:
            break;
    }
}
